#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <malloc.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>

#ifndef TRUE

#define TRUE  1
#define FALSE 0 

#endif

#define MAX_SERIAL_PORTS   20             /* Maximum number of serial ports */
#define MAX_PARALLEL_PORTS 3            /* Maximum number of parallel ports */
#define FIRSTBUTTON        0
#define NEXTBUTTON         1
#define MAX_COMMAND_LENGTH 3
#define MAX_DATA_LENGTH    64
#define OUR_SEM_PERMS      0x1B6              /* All access for any process */

typedef unsigned char uchar;
typedef unsigned long dword;

/*
   Census information structures
*/

typedef enum                                
{
   SerialAdapter,                                            /* i.e. DS1412 */ 
   ParallelAdapter,                              /* i.e. DS1410X or DS1410D */
   LastAdapter                               /* Add new adapter types above */ 
}
AdaptType;

typedef struct
{
   uchar PortType;                             /* Adapter type ROM found in */
   uchar PortNumber;                           /* Port number part found in */
}
RLSType;

typedef struct
{
   uchar PortType;             /* Type of adapter this button was read from */
   uchar PortNumber;                              /* Zero based port number */
   uchar RomData[8];                              /* Holds buttons ROM data */
}
ROMLocType;

/*
    Structure for DS1412 block data packets.
*/

typedef struct DS1412Packet
{
   uchar PacketLength;                       /* Length of data transmission */
   uchar RomCommand;                                       /* Access (0x55) */
   uchar RomData[8];                                  /* ROM data of DS1425 */
   uchar CommandLength;                          /* Number of command bytes */
   uchar CommandData[MAX_COMMAND_LENGTH];           /* Button command bytes */
   uchar DataPacketLength;                         /* Length of data packet */
   uchar DataPacket[MAX_DATA_LENGTH];                         /* Data paket */
}
DS1412PACKET, *PDS1412PACKET;
 
uchar *take_census(uchar *);                     /* API function prototypes */
uchar lock_rom(uchar *, dword);
uchar unlock_rom(uchar *);
uchar check_rom(uchar *);
uchar read_1425_sp(uchar *, uchar *);
uchar write_1425_sp(uchar *, uchar *);
uchar read_subkey(uchar, uchar *, uchar *);
uchar write_subkey(uchar, uchar *, uchar *);
uchar reprogram_subkey(uchar, uchar *, uchar *);
uchar move_block(uchar, uchar *, uchar *, uchar);
uchar read_1427_sp(uchar *, uchar *);
uchar write_1427_sp(uchar *, uchar *, uchar);
uchar copy_1427_sp(uchar *, uchar *); 
uchar read_1427_smem(uchar *, uchar *, uchar);

uchar dskeyopen(void);                     /* Access system function protos */
uchar dskeyclose(void);
uchar dowcheck(void);
uchar dssetup(uchar);                
uchar dsfirst(void);
uchar dsnext(void);
uchar dsaccess(void);
uchar dsgndtest(void);
uchar dsdatabyte(uchar);
uchar dskeyopen(void);
uchar dskeyclose(void);
uchar *dsromdata(void);

                                    /* Serial port transfer layer functions */
uchar SerialRS(uchar *RomFound, uchar PortNum, uchar FoNFlag);
uchar XmitDS1412Data(PDS1412PACKET PDS1412Packet, uchar PortNum);





